# Generated by Django 2.0.2 on 2018-06-07 13:12

from django.db import migrations, models
import shard_static.mixins


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0002_statictransmitstatus'),
    ]

    operations = [
        migrations.CreateModel(
            name='ShardStaticTransmittableB',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'abstract': False,
            },
            bases=(shard_static.mixins.ShardStaticMixin, models.Model),
        ),
    ]
